import os
import json
from oauth2client.service_account import ServiceAccountCredentials
from googleapiclient.discovery import build
from google.oauth2 import service_account

creds_json = os.environ.get("GOOGLE_CALENDAR_CREDENTIALS")
if creds_json:
    creds = service_account.Credentials.from_service_account_info(
        json.loads(creds_json))


class Calendar():

    def __init__(self, calendarId):

        self.service = build("calendar",
                             "v3",
                             credentials=creds,
                             cache_discovery=False)
        self.calendarId = calendarId

    def list_events(self, start_date):
        """
        :param start_date: The date filter for the events. Expects an ISO formatted date object
        Example          : pendulum.datetime(2020, 9, 1).isoformat()

        :return: Dictionary of event items
        """

        page_token = None
        events = []
        count = 0

        while True:
            # Remember that maxResults does not guarantee the number of results on one page
            # Use pagination instead: https://developers.google.com/calendar/v3/pagination
            # Code for generating page tokens: https://developers.google.com/calendar/v3/reference/calendarList/list#python
            events_list = self.service.events().list(
                calendarId=self.calendarId,
                pageToken=page_token,
                timeMin=start_date,
                maxResults=100,
                singleEvents=True,
                orderBy='startTime').execute()

            for event in events_list['items']:
                events.append(event)

            page_token = events_list.get('nextPageToken')

            count += 1

            print(f"Google Calendar: Fetching results from page {count} ...")

            if not page_token:
                break

        print(f"Fetched a total of {len(events)} events.")
        return events



import os
import pygsheets
import pandas as pd
from pygsheets import DataRange


class Gsheets:
    """
    Pass in a service account file path and query data stored in Google Sheets
    Example: Gsheets('SPREADSHEET').query('WORKSHEET')
    """

    def __init__(self, spreadsheet):
        self.client = pygsheets.authorize(
            service_account_env_var="GSHEETS_SERVICE_ACCOUNT")
        self.spreadsheet_key = os.getenv(spreadsheet)
        self.spreadsheet = self.client.open_by_key(self.spreadsheet_key)

    def query(self, worksheet, start_cell=None, end_cell=None):
        """
        Returns results of spreadsheet, worksheet as a pandas DataFrame
        Spreadsheet is stored as env var
        Worksheet is passed in as plain text
        """
        ws = self.spreadsheet.worksheet_by_title(worksheet)
        df = ws.get_as_df(start=start_cell, end=end_cell)
        print(
            f"{self.spreadsheet.title}.{worksheet}: Downloaded {df.shape} shape"
        )
        return df

    # If get_as_df does not work, use this instead
    def select_all_values(self, worksheet):
        return pd.DataFrame(
            self.spreadsheet.worksheet_by_title(worksheet).get_all_records())

    def write_df(self, df, worksheet, start_cell='A1'):
        self.spreadsheet.worksheet_by_title(worksheet).set_dataframe(
            df, start=start_cell)

    def write_df_with_timestamp(self, df, worksheet):
        # Create worksheet if not exists
        try:
            worksheet = self.spreadsheet.worksheet_by_title(worksheet)
        except Exception as e:
            worksheet = self.spreadsheet.add_worksheet(title=worksheet)

        # Add a timestamp to the df
        df['updated_at'] = pd.Timestamp("now")

        # Clear existing data
        worksheet.clear(start='A1')

        # Write the frame to the worksheet, adding rows if necessary
        worksheet.set_dataframe(df, start="A1", extend=True)

        NUM_ROWS = df.shape[0] + 1
        NUM_COLS = df.shape[1]

        # Resize the worksheet to the shape of the frame
        worksheet.resize(rows=NUM_ROWS, cols=NUM_COLS)

        # TODO: Delete and recreate named range does not work - Google Sheets Query will break if the range is updated
        # First try deleting the named range
        # try:
        #     worksheet.delete_named_range(worksheet.title)
        #     print(f"Deleted named range: {worksheet.title}.")
        # except Exception as e:
        #     print(f"Named range: {worksheet.title} does not exist.")

        # # Create the named range
        # worksheet.create_named_range(
        #     name=worksheet.title,
        #     start='A1',
        #     end=f"Z{NUM_ROWS}",
        #     grange=None
        # )
        # print(f"Created named range: {worksheet.title}.")

        # Create the DataRange object
        drange = DataRange(start='A1', worksheet=worksheet)

        # Add borders to the DataRange
        drange.update_borders(top=True,
                              right=True,
                              bottom=True,
                              left=True,
                              inner_horizontal=True,
                              inner_vertical=True,
                              style='SOLID')

        # Hide the worksheet
        # worksheet.hidden = True

        # Complete message
        print(
            f"Wrote {len(df)} rows to {self.spreadsheet.title}.{worksheet.title}"
        )



from google.cloud import storage


def upload_blob(bucket_name, source_file_name, destination_blob_name):
    """Uploads a file to the bucket."""
    # The ID of your GCS bucket
    # bucket_name = "your-bucket-name"
    # The path to your file to upload
    # source_file_name = "local/path/to/file"
    # The ID of your GCS object
    # destination_blob_name = "storage-object-name"

    storage_client = storage.Client()
    bucket = storage_client.bucket(bucket_name)
    blob = bucket.blob(destination_blob_name)

    # Optional: set a generation-match precondition to avoid potential race conditions
    # and data corruptions. The request to upload is aborted if the object's
    # generation number does not match your precondition. For a destination
    # object that does not yet exist, set the if_generation_match precondition to 0.
    # If the destination object already exists in your bucket, set instead a
    # generation-match precondition using its generation number.
    generation_match_precondition = 0

    blob.upload_from_filename(source_file_name, if_generation_match=generation_match_precondition)

    print(
        f"File {source_file_name} uploaded to {destination_blob_name}."
    )
